/*
 * pthread/fork-echo.c --
 *
 *      A simple program to fork processes and to wait for them.
 */

#define _POSIX_C_SOURCE 200809L

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>

static void
work(const char *msg)
{
    (void) printf("%s ", msg);
    exit(EXIT_SUCCESS);
}

int
main(int argc, char *argv[])
{
    int i, status;
    pid_t pids[argc];

    for (i = 1; i < argc; i++) {
        pids[i] = fork();
        if (pids[i] == -1) {
            fprintf(stderr, "failed to fork: %s\n", strerror(errno));
            continue;
        }
        if (pids[i] == 0) {
            work(argv[i]);
        }
    }

    for (i = 1; i < argc; i++) {
        if (pids[i] > 0) {
            (void) waitpid(pids[i], &status, 0);
        }
    }
    (void) printf("\n");

    return EXIT_SUCCESS;
}
